/// @file Decoder.hpp
/// @author Franck Dary
/// @version 1.0
/// @date 2018-08-03

#ifndef DECODER__H
#define DECODER__H

#include "TransitionMachine.hpp"
#include "Config.hpp"

/// @brief A simple object capable of using a trained TransitionMachine to process a given BD.
class Decoder
{
  private :

  /// @brief The trained TransitionMachine
  TransitionMachine & tm;
  /// @brief The current configuration of the TransitionMachine
  Config & config;
  std::map< std::string, std::map<std::string, int> > nbActionsPerClassifier;

  private :

  /// @brief Fill bd using tm, with beam serach.
  void decodeBeam();
  /// @brief Fill bd using tm, without beam search.
  void decodeNoBeam();
  /// @brief Print for each classifier, the detail of predicted actions.
  void printActionsPerClassifier(FILE * output);

  public :

  /// @brief Use tm to fill bd.
  ///
  /// At the start of the function, bd must contain the input.\n
  /// At the end of the function, bd will be terminal.
  /// @param tm The trained TransitionMachine
  /// @param config The current configuration of the TransitionMachine
  Decoder(TransitionMachine & tm, Config & config);
  /// @brief Fill bd using tm.
  void decode();
};

#endif
