/// @file Error.hpp
/// @author Franck Dary
/// @version 1.0
/// @date 2018-12-12

#ifndef ERROR__H
#define ERROR__H

#include "Classifier.hpp"
#include <vector>
#include <string>

class Error
{
  private :

  std::string prediction;
  std::string gold;
  Classifier::WeightedActions weightedActions;
  std::string type;
  int indexOfPrediction;
  int indexOfGold;
  int distanceWithGold;
  float entropy;
  int cost;
  int linkLengthPrediction;
  int linkLengthGold;
  
  public :

  Error(std::string &, std::string &, Classifier::WeightedActions &, int cost, int linkLengthPrediction, int linkLengthGold);
  bool isError() const;
  const std::string & getType() const;
  bool goldWasAtDistance(int distance) const;
  float getEntropy() const;
  float getCost() const;
  int getLinkLengthPrediction() const;
  int getLinkLengthGold() const;
};

class ErrorSequence
{
  private :

  std::vector<Error> sequence;
  std::map<std::string, bool> types;

  public :

  void add(const Error & error);
  const std::map<std::string, bool> & getTypes() const;
  const std::vector<Error> & getSequence() const;
};

class Errors
{
  private :

  std::vector<ErrorSequence> sequences;

  public :

  void newSequence();
  void add(const Error & error);
  void printStats();
};

#endif
