/// @file macaon_train.cpp
/// @author Franck Dary
/// @version 1.0
/// @date 2018-08-07

#include <cstdio>
#include <cstdlib>
#include "BD.hpp"
#include "Config.hpp"
#include "TransitionMachine.hpp"
#include "Trainer.hpp"
#include "ProgramParameters.hpp"
#include "programOptionsTemplates.hpp"

/// @brief Train a TransitionMachine to predict and add information to a structured input file, by using annotated examples.
///
/// @param argc The number of arguments given to this program.
/// @param argv[] Array of arguments given to this program.
///
/// @return 0 if there was no crash.
int main(int argc, char * argv[])
{
  loadTrainProgramParameters(argc, argv);

  if (ProgramParameters::nbTrain)
  {
    updatePaths();
    createTemplatePath();
    for (int i = 0; i < ProgramParameters::nbTrain; i++)
    {
      fprintf(stderr, "Training number %d / %d :\n", i+1, ProgramParameters::nbTrain);
      ProgramParameters::expName = ProgramParameters::baseExpName + "_" + std::to_string(i);
      updatePaths();
      createExpPath();
      Dict::deleteDicts();
      launchTraining();
    }
    removeTemplatePath();
  }
  else
  {
    updatePaths();
    ProgramParameters::newTemplatePath = ProgramParameters::templatePath;
    createExpPath();
    launchTraining();
  }

  return 0;
}

