#include "ActionSet.hpp"
#include "File.hpp"
#include "util.hpp"

ActionSet::ActionSet(const std::string & filename, bool isDynamic)
{
  this->isDynamic = isDynamic;
  hasDefaultAction = false;

  if(!isDynamic)
  {
    File file(filename, "r");
    FILE * fd = file.getDescriptor();

    char buffer[1024];

    if(fscanf(fd, "Default : %[^\n]\n", buffer) == 1)
    {
      str2index[buffer] = actions.size();
      actions.emplace_back(buffer);
      hasDefaultAction = true;
    }

    while(fscanf(fd, "%[^\n]\n", buffer) == 1)
    {
      str2index[buffer] = actions.size();
      actions.emplace_back(buffer);
    }

    this->name = getFilenameFromPath(filename);
  }
  else
  {
    this->name = filename;
  }
}

void ActionSet::printForDebug(FILE * output)
{
  fprintf(output, "ActionSet %s :\n", name.c_str());

  for(auto & action : actions)
    action.printForDebug(output);
}

int ActionSet::getActionIndex(const std::string & name)
{
  auto it = str2index.find(name);

  if(it != str2index.end())
    return it->second;

  if(!isDynamic)
  {
    if(hasDefaultAction)
      return 0;

    fprintf(stderr, "ERROR (%s) : unknown action \'%s\'. Aborting.\n", ERRINFO, name.c_str());
    printForDebug(stderr);

    exit(1);
  }

  str2index[name] = actions.size();
  actions.emplace_back(name);

  return str2index[name];
}

std::string ActionSet::getActionName(int actionIndex)
{
  if(actionIndex >= 0 && actionIndex < (int)actions.size())
  {
    return actions[actionIndex].name;
  }

  fprintf(stderr, "ERROR (%s) : invalid action index \'%d\'. Aborting.\n", ERRINFO, actionIndex);

  exit(1);

  return "";
}

Action * ActionSet::getAction(const std::string & name)
{
  return &actions[getActionIndex(name)];
}

unsigned int ActionSet::size()
{
  return actions.size();
}

